%Author:    Dimme
%Descr:     This function reports the statistics for the listening experiments:
%               -average+sdev confidence, intelligibility and quality
%               -Some other statistics
%           Based on weighted averages in which age, hearing impairment and english level are taken into account.
clear all
close all

load('Results/table_statistics.mat')

%the number of listeners
Ntot = length(find(table_statistics(:,2)>3))/15;
disp("The total number of listeners older than 40 is " + num2str(Ntot))

Ntot = length(find(table_statistics(:,2)<4))/15;
disp("The total number of listeners younger than 40 " + num2str(Ntot))

Ntot = length(find(isnan(table_statistics(:,2))))/15;
disp("The total number of listeners with unknown age is " + num2str(Ntot))
disp(" ")

%the average quality/algorithm 
AGE = table_statistics(:,2);
QUAL = table_statistics(:,6);
ALGORITHM = table_statistics(:,9);
IMPAIRED = table_statistics(:,1);
[ALGORITHM,I] = sort(ALGORITHM);
QUAL = QUAL(I);
AGE = AGE(I);
IMPAIRED = IMPAIRED(I);
for i=1:size(table_statistics,1)
    if ~isnan(QUAL(i))
        if (AGE(i) == 1 || AGE(i) == 2 || AGE(i) == 3) && IMPAIRED(i) == 2
            weighting(i) = 3;
        elseif (AGE(i) == 4) && IMPAIRED(i) == 2
            weighting(i) = 2;
        else
            weighting(i) = 1;
        end
    else 
        weighting(i) = 0;
    end
end
weighting = weighting';
INDX = find(ALGORITHM==1); INDX1 = INDX(1); INDX2 = INDX(end); 
disp("The average quality rating of NN: " + num2str(sum(QUAL(INDX1:INDX2).*weighting(INDX1:INDX2), "omitnan")/sum(weighting(INDX1:INDX2))) )
disp("The sdev of the quality rating NN: " + num2str(std(QUAL(INDX1:INDX2), weighting(INDX1:INDX2), "omitnan")))
INDX = find(ALGORITHM==2); INDX1 = INDX(1); INDX2 = INDX(end); 
disp("The average quality rating of GEV: " + num2str(sum(QUAL(INDX1:INDX2).*weighting(INDX1:INDX2), "omitnan")/sum(weighting(INDX1:INDX2))) )
disp("The sdev of the quality rating GEV: " + num2str(std(QUAL(INDX1:INDX2), weighting(INDX1:INDX2), "omitnan")))
INDX = find(ALGORITHM==3); INDX1 = INDX(1); INDX2 = INDX(end); 
disp("The average quality rating of Novel 0.1: " + num2str(sum(QUAL(INDX1:INDX2).*weighting(INDX1:INDX2), "omitnan")/sum(weighting(INDX1:INDX2))) )
disp("The sdev of the quality rating Novel 0.1: " + num2str(std(QUAL(INDX1:INDX2), weighting(INDX1:INDX2), "omitnan")))
INDX = find(ALGORITHM==4); INDX1 = INDX(1); INDX2 = INDX(end); 
disp("The average quality rating of Novel 1: " + num2str(sum(QUAL(INDX1:INDX2).*weighting(INDX1:INDX2), "omitnan")/sum(weighting(INDX1:INDX2))) )
disp("The sdev of the quality rating Novel 1: " + num2str(std(QUAL(INDX1:INDX2), weighting(INDX1:INDX2), "omitnan")))
disp(" ")

%the average intelligibility/algorithm 
ENGLISHLEVEL = table_statistics(:,4);
INTEL = table_statistics(:,7);
ALGORITHM = table_statistics(:,9);
IMPAIRED = table_statistics(:,1);

[ALGORITHM,I] = sort(ALGORITHM);
INTEL = INTEL(I);
ENGLISHLEVEL = ENGLISHLEVEL(I);
IMPAIRED = IMPAIRED(I);
for i=1:size(table_statistics,1)
    if ~isnan(INTEL(i))
        if (ENGLISHLEVEL(i)==1) && IMPAIRED(i) == 2
            weighting(i) = 1;
        elseif (ENGLISHLEVEL(i)==2) && IMPAIRED(i) == 2
            weighting(i) = 2;
        elseif (ENGLISHLEVEL(i)==3) && IMPAIRED(i) == 2
            weighting(i) = 3;
        elseif (ENGLISHLEVEL(i)==4) && IMPAIRED(i) == 2
            weighting(i) = 4;
        else
            weighting(i) = 2;
        end
    else 
        weighting(i) = 0;
    end
end
INDX = find(ALGORITHM==1); INDX1 = INDX(1); INDX2 = INDX(end); 
disp("The average intelligibility rating of NN: " + num2str(sum(INTEL(INDX1:INDX2).*weighting(INDX1:INDX2), "omitnan")/sum(weighting(INDX1:INDX2))) )
disp("The sdev of the intelligibility rating NN: " + num2str(std(INTEL(INDX1:INDX2), weighting(INDX1:INDX2), "omitnan")))
INDX = find(ALGORITHM==2); INDX1 = INDX(1); INDX2 = INDX(end); 
disp("The average intelligibility rating of GEV: " + num2str(sum(INTEL(INDX1:INDX2).*weighting(INDX1:INDX2), "omitnan")/sum(weighting(INDX1:INDX2))) )
disp("The sdev of the intelligibility rating GEV: " + num2str(std(INTEL(INDX1:INDX2), weighting(INDX1:INDX2), "omitnan")))
INDX = find(ALGORITHM==3); INDX1 = INDX(1); INDX2 = INDX(end); 
disp("The average intelligibility rating of Novel 0.1: " + num2str(sum(INTEL(INDX1:INDX2).*weighting(INDX1:INDX2), "omitnan")/sum(weighting(INDX1:INDX2))) )
disp("The sdev of the intelligibility rating Novel 0.1: " + num2str(std(INTEL(INDX1:INDX2), weighting(INDX1:INDX2), "omitnan")))
INDX = find(ALGORITHM==4); INDX1 = INDX(1); INDX2 = INDX(end); 
disp("The average intelligibility rating of Novel 1: " + num2str(sum(INTEL(INDX1:INDX2).*weighting(INDX1:INDX2), "omitnan")/sum(weighting(INDX1:INDX2))) )
disp("The sdev of the intelligibility rating Novel 1: " + num2str(std(INTEL(INDX1:INDX2), weighting(INDX1:INDX2), "omitnan")))
disp(" ")

%the average confidence/algorithm 
CONFIDENCE = table_statistics(:,5);
ALGORITHM = table_statistics(:,9);
IMPAIRED = table_statistics(:,1);

[ALGORITHM,I] = sort(ALGORITHM);
CONFIDENCE = CONFIDENCE(I);
IMPAIRED = IMPAIRED(I);
for i=1:size(table_statistics,1)
    if ~isnan(CONFIDENCE(i))
        if (AGE(i) == 1 || AGE(i) == 2 || AGE(i) == 3) && IMPAIRED(i) == 2
            weighting(i) = 3;
        elseif AGE(i)==4 && IMPAIRED(i) == 2
            weighting(i) = 2;
        else 
            weighting(i) = 1;
        end
    else 
        weighting(i) = 0;
    end
end
INDX = find(ALGORITHM==1); INDX1 = INDX(1); INDX2 = INDX(end); 
disp("The average confidence rating of NN: " + num2str(sum(CONFIDENCE(INDX1:INDX2).*weighting(INDX1:INDX2), "omitnan")/sum(weighting(INDX1:INDX2))) )
disp("The sdev of the confidence rating NN: " + num2str(std(CONFIDENCE(INDX1:INDX2), weighting(INDX1:INDX2), "omitnan")))
INDX = find(ALGORITHM==2); INDX1 = INDX(1); INDX2 = INDX(end); 
disp("The average confidence rating of GEV: " + num2str(sum(CONFIDENCE(INDX1:INDX2).*weighting(INDX1:INDX2), "omitnan")/sum(weighting(INDX1:INDX2))) )
disp("The sdev of the confidence rating GEV: " + num2str(std(CONFIDENCE(INDX1:INDX2), weighting(INDX1:INDX2), "omitnan")))
INDX = find(ALGORITHM==3); INDX1 = INDX(1); INDX2 = INDX(end); 
disp("The average confidence rating of Novel 0.1: " + num2str(sum(CONFIDENCE(INDX1:INDX2).*weighting(INDX1:INDX2), "omitnan")/sum(weighting(INDX1:INDX2))) )
disp("The sdev of the confidence rating Novel 0.1: " + num2str(std(CONFIDENCE(INDX1:INDX2), weighting(INDX1:INDX2), "omitnan")))
INDX = find(ALGORITHM==4); INDX1 = INDX(1); INDX2 = INDX(end); 
disp("The average confidence rating of Novel 1: " + num2str(sum(CONFIDENCE(INDX1:INDX2).*weighting(INDX1:INDX2), "omitnan")/sum(weighting(INDX1:INDX2))) )
disp("The sdev of the confidence rating Novel 1: " + num2str(std(CONFIDENCE(INDX1:INDX2), weighting(INDX1:INDX2), "omitnan")))
disp(" ")


