%Author:    Dimme
%Descr:     This function gives the results per virtual source direction (A up to K). 
%           -The mean direction is plotted and the individual directions are plotted.
%           -The spherical variance is printed, 
%           -the error wrt target in azimuth and elevation are printed,
%           -and the error in terms of a single angle is printed.

clear all
close all 
clc

load('Results/table_loc.mat')
table_locMeas = table_loc(table_loc(:,7) ~= 3,:); %Remove training signals

%Select a virtual source (in paper labels)
VS = "K";   %paper notation

%Select the virtual source (in earlier notation) corresponding to the selected source in paper notation. Convert to number as given in Column 6 of the table.
VSmapping = ["A", "AC", "B", "C", "D", "R", "Q", "S", "U", "W", "P"; ...  
              "B", "G", "I", "K", "E", "C", "H", "F", "J", "A", "D"];         
indx1 = find(VSmapping(2,:)==VS);   
VS = find(dataVec == VSmapping(1,indx1));

%compute mean and standard error on training data
algorithm = 1;
[meanDirCart_1, meanDirSphere_1, spread_1, big_table1]  = fnc_errorTrain(table_locMeas, algorithm, VS);
algorithm = 2;
[meanDirCart_2, meanDirSphere_2, spread_2, big_table2]  = fnc_errorTrain(table_locMeas, algorithm, VS);
algorithm = 3;
[meanDirCart_3, meanDirSphere_3, spread_3, big_table3]  = fnc_errorTrain(table_locMeas, algorithm, VS);
algorithm = 4;
[meanDirCart_4, meanDirSphere_4, spread_4, big_table4]  = fnc_errorTrain(table_locMeas, algorithm, VS);

distALGglobe = 180/pi*1*acos((dot(meanDirCart_1(1,:), meanDirCart_1(2,:))));
error = 180/pi*(meanDirSphere_1(1,2:3)-meanDirSphere_1(2,2:3));
if error(1)<-180
    error(1) = error(1)+360;
end
disp("error NN to target [degree]: " + num2str(error));
disp("error NN to target (single param) [degree]: " + num2str(distALGglobe));
%disp("error NN to epsilon [degree]: " + num2str(180/pi*(meanDirSphere_1(1,2:3)-meanDirSphere_1(3,2:3))));
disp("spread NN: " + num2str(spread_1, "%.2f"));

disp(" ")
distALGglobe = 180/pi*1*acos((dot(meanDirCart_2(1,:), meanDirCart_2(2,:))));
error = 180/pi*(meanDirSphere_2(1,2:3)-meanDirSphere_2(2,2:3));
if error(1)<-180
    error(1) = error(1)+360;
end
disp("error GEV to target [degree]: " + num2str(error));
disp("error GEV to target (single param) [degree]: " + num2str(distALGglobe));
%disp("error GEV to epsilon [degree]: " + num2str(180/pi*(meanDirSphere_2(1,2:3)-meanDirSphere_2(3,2:3))));
disp("spread GEV: " + num2str(spread_2, "%.2f"));

disp(" ")
distALGglobe = 180/pi*1*acos((dot(meanDirCart_3(1,:), meanDirCart_3(2,:))));
error = 180/pi*(meanDirSphere_3(1,2:3)-meanDirSphere_3(2,2:3));
if error(1)<-180
    error(1) = error(1)+360;
end
disp("error Novel 0.1 to target [degree]: " + num2str(error));
disp("error Novel 0.1 to target (single param) [degree]: " + num2str(distALGglobe));
%disp("error Novel 0.1 to epsilon [degree]: " + num2str(180/pi*(meanDirSphere_3(1,2:3)-meanDirSphere_3(3,2:3))));
disp("spread Novel 0.1: " + num2str(spread_3, "%.2f"));

disp(" ")
distALGglobe = 180/pi*1*acos((dot(meanDirCart_4(1,:), meanDirCart_4(2,:))));
error = 180/pi*(meanDirSphere_4(1,2:3)-meanDirSphere_4(2,2:3));
if error(1)<-180
    error(1) = error(1)+360;
end
disp("error Novel 1 to target [degree]: " + num2str(error));
disp("error Novel 1 to target (single param) [degree]: " + num2str(distALGglobe));
%disp("error Novel 1 to epsilon [degree]: " + num2str(180/pi*(meanDirSphere_4(1,2:3)-meanDirSphere_4(3,2:3))));
disp("spread Novel 1: " + num2str(spread_4, "%.2f"));

%plot 
figure
N1 = size(big_table1,1);
N2 = size(big_table2,1);
N3 = size(big_table3,1);
N4 = size(big_table4,1);
quiver3(zeros(N1,1), zeros(N1,1), zeros(N1,1), big_table1(:,2), big_table1(:,3), big_table1(:,4))
hold on
quiver3(zeros(N2,1), zeros(N2,1), zeros(N2,1), big_table2(:,2), big_table2(:,3), big_table2(:,4))
quiver3(zeros(N3,1), zeros(N3,1), zeros(N3,1), big_table3(:,2), big_table3(:,3), big_table3(:,4))
quiver3(zeros(N4,1), zeros(N4,1), zeros(N4,1), big_table4(:,2), big_table4(:,3), big_table4(:,4))
ref = 1.1*[ big_table1(1,8), big_table1(1,9), big_table1(1,10);];
quiver3(zeros(1,1), zeros(1,1), zeros(1,1), ref(:,1), ref(:,2),ref(:,3), 'color', 'black', 'linewidth', 3)
xlabel('x')
ylabel('y')
zlabel('z') 
legend('NN', 'GEV', 'NOV0.1', 'NOV1', 'target');
title("results " + VSmapping(2,indx1))

figure
quiver3(0,0, 0, meanDirCart_1(1,1), meanDirCart_1(1,2), meanDirCart_1(1,3))
hold on
quiver3(0,0, 0, meanDirCart_2(1,1), meanDirCart_2(1,2), meanDirCart_2(1,3))
quiver3(0,0, 0, meanDirCart_3(1,1), meanDirCart_3(1,2), meanDirCart_3(1,3))
quiver3(0,0, 0, meanDirCart_4(1,1), meanDirCart_4(1,2), meanDirCart_4(1,3))
refTar = meanDirCart_1(2,:);
refNN = meanDirCart_1(3,:);
quiver3(zeros(1,1), zeros(1,1), zeros(1,1), refTar(1), refTar(2),refTar(3), 'color', 'black', 'linewidth', 2)
quiver3(zeros(1,1), zeros(1,1), zeros(1,1), refNN(1), refNN(2),refNN(3), 'color', 'black', 'linewidth', 2, 'linestyle', '--')
xlabel('x')
ylabel('y')
zlabel('z') 
legend('NN', 'GEV', 'NOV0.1', 'NOV1', 'target', 'actual NN');
title("Mean results " + VSmapping(2,indx1))

function  [meanDirCart, meanDirSphere, spread, big_table]  = fnc_errorTrain(big_table, algorithm, VS)
    %output:    meanDirCart (cartesian coordinates,  (x, y, z) [measured; target; nearest neighbour]);
    %           meanDirSpher (spherical coordinates, (r, theta, phi) [measured; target; nearest neighbour]);
    %           spread -> spherical variance
    
    %pick algorithm, normalise vector lengths
    big_table = big_table(big_table(:,6)==algorithm,:);
    big_table = big_table(big_table(:,5)==VS,:);
    big_table(:, 2:4) = big_table(:, 2:4)./vecnorm(big_table(:, 2:4),2,2);
    big_table(:, 8:10) = big_table(:, 8:10)./vecnorm(big_table(:, 8:10),2,2);
    big_table(:, 11:13) = big_table(:, 11:13)./vecnorm(big_table(:, 11:13),2,2);

    %compute mean direction in cartesian and in spherical coordinates
    %compute weighting:
    AGE = big_table(:,end);
    IMPAIRED = big_table(:,end-1);
    for i=1:size(big_table,1)
        if ~isnan(AGE(i))
            if (AGE(i) == 1 || AGE(i) == 2 || AGE(i) == 3) && IMPAIRED(i) == 2
                weighting(i) = 3;
            elseif AGE(i)==4 && IMPAIRED(i) == 2
                weighting(i) = 2;
            else
                weighting(i) = 1;
            end
        else 
            weighting(i) = 0;
        end
    end
    weighting = length(weighting)*weighting'/sum(weighting);
    %weighting = ones(size(weighting));

    meanDirCart = [mean(weighting.*big_table(:,2:4), 'omitnan'); mean(weighting.*big_table(:,8:10), 'omitnan'); mean(weighting.*big_table(:,11:13), 'omitnan')];
    for i=1:3
        [meanDirSphere(i,1),  meanDirSphere(i,2),  meanDirSphere(i,3)] = fnc_cart2sph(meanDirCart(i,1), meanDirCart(i,2), meanDirCart(i,3));
    end
    meanDirCart = meanDirCart./vecnorm(meanDirCart,2,2);

    %estimate spread for user variables, see [1]
    Rbar = meanDirSphere(1,1);
    spread = 1-Rbar^2;  %Mardia 1999, directional statistics pp. 164-166 and in particular the interpretation of Eq. 9.2.11

    %Alternative for sprea: the kappa parameter of the von mises fisher distribution. See  the wikipedia page 
    % spread = fnc_newton(2, Rbar); %in case of concentration parameter
    % function spread = fnc_newton(Niter, Rbar)
    %     p = 3;
    %     spread = Rbar*(p-Rbar^2)/(1-Rbar^2);
    %     for iterCount=1:Niter
    %         ApS = besseli(p/2,spread)/besseli(p/2-1, spread);
    %         spread = spread - (ApS - Rbar)/(1 - ApS^2 - (p-1)/spread * ApS) ;
    %     end
    % end
end

