%Author:    Dimme
%Descr:     This function plots the responses of the users to the training signals. 

clear all
close all

load('Results/table_loc.mat')

table_locTrain = table_loc(table_loc(:,7) == 3,:); %consider only the training examples

%compute mean and standard error on training data
algorithm = 1;
[meanDirCart_NN1, meanDirSphere_NN1, spread_NN1, big_tableNN1]  = fnc_errorTrain(table_locTrain, algorithm);
algorithm = 2;
[meanDirCart_NN2, meanDirSphere_NN2, spread_NN2, big_tableNN2]  = fnc_errorTrain(table_locTrain, algorithm);
algorithm = 3;
[meanDirCart_NN3, meanDirSphere_NN3, spread_NN3, big_tableNN3]  = fnc_errorTrain(table_locTrain, algorithm);

%plot 
quiver3(zeros(52,1), zeros(52,1), zeros(52,1), big_tableNN1(:,2), big_tableNN1(:,3), big_tableNN1(:,4))
hold on
quiver3(zeros(52,1), zeros(52,1), zeros(52,1), big_tableNN2(:,2), big_tableNN2(:,3), big_tableNN2(:,4))
quiver3(zeros(52,1), zeros(52,1), zeros(52,1), big_tableNN3(:,2), big_tableNN3(:,3), big_tableNN3(:,4))
ref = 1.1*[ big_tableNN1(1,8), big_tableNN1(1,9), big_tableNN1(1,10);...
        big_tableNN2(1,8), big_tableNN2(1,9), big_tableNN2(1,10); ...
        big_tableNN3(1,8), big_tableNN3(1,9), big_tableNN3(1,10) ];
quiver3(zeros(3,1), zeros(3,1), zeros(3,1), ref(:,1), ref(:,2),ref(:,3), 'color', 'black', 'linewidth', 3)
xlabel('x')
ylabel('y')
zlabel('z') 
title('results nearest neighbour - training')
legend('Training signal 1', 'Training signal 2', 'Training signal 3', 'Target directions (physical loudspeakers)')

function  [meanDirCart, meanDirSphere, spread, big_tableTrain]  = fnc_errorTrain(big_tableTrain, algorithm)
    %output:    meanDirCart (cartesian coordinates,  (x, y, z) [measured; target; nearest neighbour]);
    %           meanDirSpher (spherical coordinates, (r, theta, phi) [measured; target; nearest neighbour]);
    %           spread (see [1],  (kappa) measured);
    %   
    %           [1] https://en.wikipedia.org/wiki/Von_Mises%E2%80%93Fisher_distribution#Mean_direction
    
    %pick algorithm, normalise vector lengths
    big_tableTrain = big_tableTrain(big_tableTrain(:,6)==algorithm,:);
    big_tableTrain(:, 2:4) = big_tableTrain(:, 2:4)./vecnorm(big_tableTrain(:, 2:4),2,2);
    big_tableTrain(:, 8:10) = big_tableTrain(:, 8:10)./vecnorm(big_tableTrain(:, 8:10),2,2);
    big_tableTrain(:, 11:13) = big_tableTrain(:, 11:13)./vecnorm(big_tableTrain(:, 11:13),2,2);

    %compute mean direction in cartesian and in spherical coordinates
    %compute weighting:
    AGE = big_tableTrain(:,end);
    IMPAIRED = big_tableTrain(:,end-1);
    for i=1:size(big_tableTrain,1)
        if ~isnan(AGE(i))
            if (AGE(i) == 1 || AGE(i) == 2 || AGE(i) == 3) && IMPAIRED(i) == 2
                weighting(i) = 3;
            elseif AGE(i)==4 && IMPAIRED(i) == 2
                weighting(i) = 2;
            else 
                weighting(i) = 1;
            end
        else 
            weighting(i) = 0;
        end
    end
    weighting = length(weighting)*weighting'/sum(weighting);
    %weighting = ones(size(weighting));

    meanDirCart = [mean(weighting.*big_tableTrain(:,2:4), 'omitnan'); mean(weighting.*big_tableTrain(:,8:10), 'omitnan'); mean(weighting.*big_tableTrain(:,11:13), 'omitnan')];
    for i=1:3
        [meanDirSphere(i,1),  meanDirSphere(i,2),  meanDirSphere(i,3)] = fnc_cart2sph(meanDirCart(i,1), meanDirCart(i,2), meanDirCart(i,3));
    end
    meanDirCart = meanDirCart./vecnorm(meanDirCart,2,2);

    %estimate spread for user variables, see [1]
    Rbar = meanDirSphere(1,1);
    spread = fnc_newton(2, Rbar);
    function spread = fnc_newton(Niter, Rbar)
        p = 3;
        spread = Rbar*(p-Rbar^2)/(1-Rbar^2);
        for iterCount=1:Niter
            ApS = besseli(p/2,spread)/besseli(p/2-1, spread);
            spread = spread - (ApS - Rbar)/(1 - ApS^2 - (p-1)/spread * ApS) ;
        end
    end
end

